==> fred_ferda_verification-1.2.0-py3-none-any.whl: ferda_verification/api.py <==
"""Ferda verification API."""
import asyncio
from typing import List, Optional

from asgiref.sync import sync_to_async
from django.http import HttpRequest as Request
from ferda.api import Error, InvalidDataError, check_permissions
from ferda.constants import LoggerResult
from ferda.settings import SETTINGS
from ferda.utils import await_with_semaphore
from fred_types import ContactHistoryId, ContactId
from grill import LogProperty, Properties
from ninja import Query, Router, Schema
from verifier import (
    ContactData,
    PaginatedList,
    Verification,
    VerificationId,
    VerificationResult,
    VerificationStatus,
)
from verifier.exceptions import (
    ContactDoesNotExist,
    IncorrectStatus,
    InvalidData,
    VerificationDoesNotExist,
)

from .backend import VERIFICATION
from .constants import LogEntryType, LogReferenceType
from .utils import create_log_entry, get_contact_last_info

router = Router(tags=["verification"])


class ListQuery(Schema):
    """Verification list query."""

    contact_id: Optional[ContactId] = None
    statuses: Optional[List[VerificationStatus]] = None
    manual_check_request: Optional[bool] = None
    page_size: Optional[int] = None
    page_token: Optional[str] = None


class VerificationWithContactHandle(Verification):
    """Verification with added contact handle."""

    contact_handle: str


@router.get("/", response=PaginatedList[VerificationWithContactHandle])
async def list(request: Request, query: ListQuery = Query(None)):  # type: ignore[no-untyped-def]
    """List verifications."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_view"])

    properties: Properties = {}
    if query.contact_id is not None:
        properties["contact_id"] = query.contact_id
    if query.statuses is not None:
        properties["statuses"] = query.statuses
    if query.manual_check_request is not None:
        properties["manual_check_request"] = str(query.manual_check_request)
    if query.page_size is not None:
        properties["page_size"] = str(query.page_size)
    if query.page_token is not None:
        properties["page_token"] = query.page_token
    log_entry = await create_log_entry(
        request, LogEntryType.VERIFICATION_LIST, properties=properties
    )
    with log_entry:
        verification_list = await VERIFICATION.list(
            contact_id=query.contact_id,
            statuses=query.statuses,
            manual_check_request=query.manual_check_request,
            page_size=query.page_size,
            page_token=query.page_token,
        )
        log_entry.references[LogReferenceType.VERIFICATION] = [
            verification.verification_id for verification in verification_list.results
        ]
        log_entry.result = LoggerResult.SUCCESS
        semaphore = asyncio.Semaphore(SETTINGS.concurrent_grpc_calls_limit)
        contacts = await asyncio.gather(
            *[
                await_with_semaphore(
                    get_contact_last_info(verification.contact_id), semaphore=semaphore
                )
                for verification in verification_list.results
            ]
        )
        result = PaginatedList[VerificationWithContactHandle](
            results=[
                VerificationWithContactHandle(
                    **verification.dict(), contact_handle=contacts[i].contact_handle
                )
                for i, verification in enumerate(verification_list.results)
            ],
            pagination=verification_list.pagination,
        )
        return result


@router.post("/", response={200: Verification, 404: Error})
async def create(request: Request, contact_id: ContactId):  # type: ignore[no-untyped-def]
    """Create new verification."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_create"])

    log_entry = await create_log_entry(
        request, LogEntryType.VERIFICATION_CREATE, properties={"contact_id": contact_id}
    )
    with log_entry:
        try:
            verification = await VERIFICATION.create(contact_id=contact_id)
            log_entry.references = {
                LogReferenceType.VERIFICATION: verification.verification_id,
                LogReferenceType.CONTACT: contact_id,
            }
            log_entry.result = LoggerResult.SUCCESS
            return verification
        except ContactDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Contact not found")


@router.get("/{verification_id}/", response={200: Verification, 404: Error})
async def get(request: Request, verification_id: VerificationId):  # type: ignore[no-untyped-def]
    """Get verification detail."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_view"])

    log_entry = await create_log_entry(
        request, LogEntryType.VERIFICATION_GET, properties={"verification_id": verification_id}
    )
    with log_entry:
        try:
            verification = await VERIFICATION.get(verification_id)
            log_entry.references = {
                LogReferenceType.VERIFICATION: verification.verification_id,
                LogReferenceType.CONTACT: verification.contact_id,
            }
            log_entry.result = LoggerResult.SUCCESS
            return verification
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")


@router.put("/{verification_id}/cancel/", response={200: None, 404: Error, 409: Error})
async def cancel(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId
):
    """Cancel opened verification."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_cancel"])

    log_entry = await create_log_entry(
        request, LogEntryType.VERIFICATION_CANCEL, properties={"verification_id": verification_id}
    )
    with log_entry:
        try:
            await VERIFICATION.cancel(verification_id)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.put("/{verification_id}/approve_futile/", response={200: None, 404: Error, 409: Error})
async def approve_futile(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId
):
    """Approve that verification is futile.

    This may lead to irreversible changes like domain deletion.
    """
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_approve_futile"]
    )

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_APPROVE_FUTILE,
        properties={"verification_id": verification_id},
    )
    with log_entry:
        try:
            await VERIFICATION.approve_futile(verification_id)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.put("/{verification_id}/delay_status/", response={200: None, 404: Error, 409: Error})
async def delay_status(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId, status: VerificationStatus, delay: int
):
    """Delay verification status.

    Delay is number of seconds. Negative values are possible to shorten status timeout.
    """
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_delay_status"]
    )

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_DELAY_STATUS,
        properties={
            "verification_id": verification_id,
            "status": status.value,
            "delay": str(delay),
        },
    )
    with log_entry:
        try:
            await VERIFICATION.delay_status(verification_id, status=status, delay=delay)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.get("/{verification_id}/results/", response={200: List[VerificationResult], 404: Error})
async def list_verification_results(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId
):
    """List verification data comparison results."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_view"])

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_LIST_RESULTS,
        properties={"verification_id": verification_id},
    )
    with log_entry:
        try:
            results = await VERIFICATION.list_verification_results(verification_id)
            log_entry.references = {
                LogReferenceType.VERIFICATION: verification_id,
            }
            log_entry.result = LoggerResult.SUCCESS
            return results
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")


class ResolveManualCheckResult(Schema):
    """Resolve manual check request body.

    Attributes:
        verified: Whether provided data are verified or not.
        contact_data: Contact data being manually checked.
        contact_history_id: Contact history id against which were `contact_data` checked.
    """

    verified: bool
    contact_data: ContactData
    contact_history_id: ContactHistoryId

    @property
    def log_properties(self) -> Properties:
        """Get resolve manual check request log properties."""
        properties: Properties = {
            "verified": str(self.verified),
            "contact_history_id": self.contact_history_id,
        }
        if self.contact_data.name:
            properties["name"] = self.contact_data.name
        if self.contact_data.organization:
            properties["organization"] = self.contact_data.organization
        if self.contact_data.birthdate:
            properties["birthdate"] = self.contact_data.birthdate.isoformat()
        if self.contact_data.company_registration_number:
            properties[
                "company_registration_number"
            ] = self.contact_data.company_registration_number
        if self.contact_data.address:
            properties["address"] = LogProperty(
                "",
                children={
                    "company": self.contact_data.address.company or "",
                    "street": ", ".join(self.contact_data.address.street),
                    "city": self.contact_data.address.city or "",
                    "state_or_province": self.contact_data.address.state_or_province or "",
                    "postal_code": self.contact_data.address.postal_code or "",
                    "country_code": self.contact_data.address.country_code or "",
                },
            )
        if self.contact_data.emails:
            properties["emails"] = ", ".join(self.contact_data.emails)
        return properties


@router.put(
    "/{verification_id}/resolve_manual_check/",
    response={200: None, 400: InvalidDataError, 404: Error, 409: Error},
)
async def resolve_manual_check(  # type: ignore[no-untyped-def]
    request: Request,
    verification_id: VerificationId,
    result: ResolveManualCheckResult,
):
    """Resolve manual check request."""
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_resolve_manual_check"]
    )

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_RESOLVE_MANUAL_CHECK,
        properties={"verification_id": verification_id, **result.log_properties},
    )
    with log_entry:
        try:
            await VERIFICATION.resolve_manual_check(
                verification_id,
                verified=result.verified,
                contact_data=result.contact_data,
                contact_history_id=result.contact_history_id,
            )
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except InvalidData as err:
            log_entry.result = LoggerResult.FAIL
            return 400, InvalidDataError(detail="Invalid data", fields=err.fields)
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.post(
    "/{verification_id}/resend_message/",
    response={200: None, 400: InvalidDataError, 404: Error, 409: Error},
)
async def resend_message(  # type: ignore[no-untyped-def]
    request: Request,
    verification_id: VerificationId,
    message_type: str,
):
    """Resend message."""
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_resend_message"]
    )
    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_RESEND_MESSAGE,
        properties={
            "verification_id": verification_id,
            "message_type": message_type,
        },
    )
    with log_entry:
        try:
            await VERIFICATION.resend_message(verification_id, message_type=message_type)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except InvalidData as err:
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.FAIL
            return 400, InvalidDataError(detail="Invalid data", fields=err.fields)
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")

==> fred_ferda_verification-1.6.0rc4-py3-none-any.whl: ferda_verification/api.py <==
"""Ferda verification API."""

import asyncio
from typing import List, Optional

from asgiref.sync import sync_to_async
from django.http import HttpRequest as Request
from ferda.api import Error, InvalidDataError, check_permissions
from ferda.constants import LoggerResult
from ferda.settings import SETTINGS
from ferda.utils import await_with_semaphore
from fred_types import ContactHistoryId, ContactId, DomainId
from grill import LogProperty, Properties
from ninja import Query, Router, Schema
from verifier import (
    ContactData,
    PaginatedList,
    Verification,
    VerificationId,
    VerificationResult,
    VerificationStatus,
)
from verifier.exceptions import (
    ContactDoesNotExist,
    IncorrectStatus,
    InvalidData,
    VerificationAlreadyExists,
    VerificationDoesNotExist,
)

from .backend import VERIFICATION
from .constants import LogEntryType, LogReferenceType
from .utils import create_log_entry, get_contact_last_info

router = Router(tags=["verification"])


class ListQuery(Schema):
    """Verification list query."""

    contact_id: Optional[ContactId] = None
    domain_id: Optional[DomainId] = None
    statuses: Optional[List[VerificationStatus]] = None
    manual_check_request: Optional[bool] = None
    page_size: Optional[int] = None
    page_token: Optional[str] = None


class VerificationWithContactHandle(Verification):
    """Verification with added contact handle."""

    contact_handle: str


@router.get("/", response=PaginatedList[VerificationWithContactHandle])
async def list(request: Request, query: ListQuery = Query(None)):  # type: ignore[no-untyped-def]
    """List verifications."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_view"])

    properties: Properties = {}
    if query.contact_id is not None:
        properties["contact_id"] = query.contact_id
    if query.domain_id is not None:
        properties["domain_id"] = query.domain_id
    if query.statuses is not None:
        properties["statuses"] = query.statuses
    if query.manual_check_request is not None:
        properties["manual_check_request"] = str(query.manual_check_request)
    if query.page_size is not None:
        properties["page_size"] = str(query.page_size)
    if query.page_token is not None:
        properties["page_token"] = query.page_token
    log_entry = await create_log_entry(
        request, LogEntryType.VERIFICATION_LIST, properties=properties
    )
    with log_entry:
        verification_list = await VERIFICATION.list(
            contact_id=query.contact_id,
            domain_id=query.domain_id,
            statuses=query.statuses,
            manual_check_request=query.manual_check_request,
            page_size=query.page_size,
            page_token=query.page_token,
        )
        log_entry.references[LogReferenceType.VERIFICATION] = [
            verification.verification_id for verification in verification_list.results
        ]
        log_entry.result = LoggerResult.SUCCESS
        semaphore = asyncio.Semaphore(SETTINGS.concurrent_grpc_calls_limit)
        contacts = await asyncio.gather(
            *[
                await_with_semaphore(
                    get_contact_last_info(verification.contact_id), semaphore=semaphore
                )
                for verification in verification_list.results
            ]
        )
        result = PaginatedList[VerificationWithContactHandle](
            results=[
                VerificationWithContactHandle(
                    **verification.dict(), contact_handle=contacts[i].contact_handle
                )
                for i, verification in enumerate(verification_list.results)
            ],
            pagination=verification_list.pagination,
        )
        return result


@router.post("/", response={200: Verification, 400: Error, 404: Error})
async def create(  # type: ignore[no-untyped-def]
    request: Request,
    contact_id: ContactId,
    reason: str = "",
    reason_url: str = "",
):
    """Create new verification."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_create"])

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_CREATE,
        properties={"contact_id": contact_id, "reason": reason, "reason_url": reason_url},
    )
    with log_entry:
        try:
            verification = await VERIFICATION.create(
                contact_id=contact_id,
                reason=reason,
                reason_url=reason_url,
            )
            log_entry.references = {
                LogReferenceType.VERIFICATION: verification.verification_id,
                LogReferenceType.CONTACT: contact_id,
            }
            log_entry.result = LoggerResult.SUCCESS
            return verification
        except ContactDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Contact not found")
        except VerificationAlreadyExists:
            log_entry.result = LoggerResult.FAIL
            return 400, Error(detail="Verification already exists")


@router.get("/{verification_id}/", response={200: Verification, 404: Error})
async def get(request: Request, verification_id: VerificationId):  # type: ignore[no-untyped-def]
    """Get verification detail."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_view"])

    log_entry = await create_log_entry(
        request, LogEntryType.VERIFICATION_GET, properties={"verification_id": verification_id}
    )
    with log_entry:
        try:
            verification = await VERIFICATION.get(verification_id)
            log_entry.references = {
                LogReferenceType.VERIFICATION: verification.verification_id,
                LogReferenceType.CONTACT: verification.contact_id,
            }
            log_entry.result = LoggerResult.SUCCESS
            return verification
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")


@router.put("/{verification_id}/cancel/", response={200: None, 404: Error, 409: Error})
async def cancel(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId
):
    """Cancel opened verification."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_cancel"])

    log_entry = await create_log_entry(
        request, LogEntryType.VERIFICATION_CANCEL, properties={"verification_id": verification_id}
    )
    with log_entry:
        try:
            await VERIFICATION.cancel(verification_id)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.put(
    "/{verification_id}/approve_futile/", response={200: Verification, 404: Error, 409: Error}
)
async def approve_futile(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId
):
    """Approve that verification is futile.

    This may lead to irreversible changes like domain deletion.
    """
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_approve_futile"]
    )

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_APPROVE_FUTILE,
        properties={"verification_id": verification_id},
    )
    with log_entry:
        try:
            verification = await VERIFICATION.approve_futile(verification_id)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
            return verification
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.put(
    "/{verification_id}/approve_manual/", response={200: Verification, 404: Error, 409: Error}
)
async def approve_manual(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId
):
    """Approve that verification is transferred to fully manual process."""
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_approve_manual"]
    )

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_APPROVE_MANUAL,
        properties={"verification_id": verification_id},
    )
    with log_entry:
        try:
            verification = await VERIFICATION.approve_manual(verification_id)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
            return verification
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.put(
    "/{verification_id}/delay_status/", response={200: Verification, 404: Error, 409: Error}
)
async def delay_status(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId, status: VerificationStatus, delay: int
):
    """Delay verification status.

    Delay is number of seconds. Negative values are possible to shorten status timeout.
    """
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_delay_status"]
    )

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_DELAY_STATUS,
        properties={
            "verification_id": verification_id,
            "status": status.value,
            "delay": str(delay),
        },
    )
    with log_entry:
        try:
            verification = await VERIFICATION.delay_status(
                verification_id, status=status, delay=delay
            )
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
            return verification
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.get("/{verification_id}/results/", response={200: List[VerificationResult], 404: Error})
async def list_verification_results(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId
):
    """List verification data comparison results."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_view"])

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_LIST_RESULTS,
        properties={"verification_id": verification_id},
    )
    with log_entry:
        try:
            results = await VERIFICATION.list_verification_results(verification_id)
            log_entry.references = {
                LogReferenceType.VERIFICATION: verification_id,
            }
            log_entry.result = LoggerResult.SUCCESS
            return results
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")


class ResolveManualCheckResult(Schema):
    """Resolve manual check request body.

    Attributes:
        verified: Whether provided data are verified or not.
        contact_data: Contact data being manually checked.
        contact_history_id: Contact history id against which were `contact_data` checked.
    """

    verified: bool
    contact_data: ContactData
    contact_history_id: ContactHistoryId

    @property
    def log_properties(self) -> Properties:
        """Get resolve manual check request log properties."""
        properties: Properties = {
            "verified": str(self.verified),
            "contact_history_id": self.contact_history_id,
        }
        if self.contact_data.name:
            properties["name"] = self.contact_data.name
        if self.contact_data.organization:
            properties["organization"] = self.contact_data.organization
        if self.contact_data.birthdate:
            properties["birthdate"] = self.contact_data.birthdate.isoformat()
        if self.contact_data.company_registration_number:
            properties["company_registration_number"] = (
                self.contact_data.company_registration_number
            )
        if self.contact_data.address:
            properties["address"] = LogProperty(
                "",
                children={
                    "company": self.contact_data.address.company or "",
                    "street": ", ".join(self.contact_data.address.street),
                    "city": self.contact_data.address.city or "",
                    "state_or_province": self.contact_data.address.state_or_province or "",
                    "postal_code": self.contact_data.address.postal_code or "",
                    "country_code": self.contact_data.address.country_code or "",
                },
            )
        if self.contact_data.emails:
            properties["emails"] = ", ".join(self.contact_data.emails)
        return properties


@router.put(
    "/{verification_id}/resolve_manual_check/",
    response={200: None, 400: InvalidDataError, 404: Error, 409: Error},
)
async def resolve_manual_check(  # type: ignore[no-untyped-def]
    request: Request,
    verification_id: VerificationId,
    result: ResolveManualCheckResult,
):
    """Resolve manual check request."""
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_resolve_manual_check"]
    )

    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_RESOLVE_MANUAL_CHECK,
        properties={"verification_id": verification_id, **result.log_properties},
    )
    with log_entry:
        try:
            await VERIFICATION.resolve_manual_check(
                verification_id,
                verified=result.verified,
                contact_data=result.contact_data,
                contact_history_id=result.contact_history_id,
            )
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except InvalidData as err:
            log_entry.result = LoggerResult.FAIL
            return 400, InvalidDataError(detail="Invalid data", fields=err.fields)
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.post(
    "/{verification_id}/resend_message/",
    response={200: None, 400: InvalidDataError, 404: Error, 409: Error},
)
async def resend_message(  # type: ignore[no-untyped-def]
    request: Request,
    verification_id: VerificationId,
    message_type: str,
):
    """Resend message."""
    await sync_to_async(check_permissions)(
        request, permissions=["ferda_verification.can_resend_message"]
    )
    log_entry = await create_log_entry(
        request,
        LogEntryType.VERIFICATION_RESEND_MESSAGE,
        properties={
            "verification_id": verification_id,
            "message_type": message_type,
        },
    )
    with log_entry:
        try:
            await VERIFICATION.resend_message(verification_id, message_type=message_type)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except InvalidData as err:
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.FAIL
            return 400, InvalidDataError(detail="Invalid data", fields=err.fields)
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")


@router.put("/{verification_id}/resume/", response={200: None, 404: Error, 409: Error})
async def resume(  # type: ignore[no-untyped-def]
    request: Request, verification_id: VerificationId
):
    """Resume processing of paused verification."""
    await sync_to_async(check_permissions)(request, permissions=["ferda_verification.can_resume"])

    log_entry = await create_log_entry(
        request, LogEntryType.VERIFICATION_RESUME, properties={"verification_id": verification_id}
    )
    with log_entry:
        try:
            await VERIFICATION.resume(verification_id)
            log_entry.references = {LogReferenceType.VERIFICATION: verification_id}
            log_entry.result = LoggerResult.SUCCESS
        except VerificationDoesNotExist:
            log_entry.result = LoggerResult.FAIL
            return 404, Error(detail="Verification not found")
        except IncorrectStatus:
            log_entry.result = LoggerResult.FAIL
            return 409, Error(detail="Incorrect verification status")
